/******************************************************************************
 *                                                                            *
 * File:   cpuspeed.c         Version  1.10            Date: 1996-08-25       *
 *                                                                            *
 * Copyright (C) 1993-1996 by kostis@acm.org (Kosta Kostis)                   *
 * This program may be used free of charge at your own risk.                  *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *        - figure out CPU speed in MHz                                       *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * History:                                                                   *
 *     1996-08-25: KK V1.10                                                   *
 *        - added support for Pentium                                         *
 *     1993-09-05: KK V1.00                                                   *
 *        - initial coding                                                    *
 *                                                                            *
 *****************************************************************************/

#include "cpuid.h"

/******************************************************************************

	Number of cycles to execute the repeated instruction in asm code.
	Cycles for the 8086, 80186, 80286, i386, i486 and Pentium processors
	Time for the 186 wasn't listed, and I'm just guessing

 *****************************************************************************/

int	CPUspeed (void)
{	
	int		CPU ;
	unsigned long	ticks ;
	unsigned long	cycles ;
	unsigned long	cps ;
	static	int	processor_cycles	[] =
	{
		 165,   165,    25,  103,   42,      42,         42
	} ;
	/*	IDIV CX for 8086/80186/80286, BSF ECX,EAX for i386/i486
		8086, 80186, 80286, i386, i486, Pentium, PentiumPro
		0     1      2      3     4     5        6 */

	CPU = CPUid () ;
	if (CPU >= P_i386)
		ticks = (unsigned long) Time386 () ;
	else
		ticks = (unsigned long) Time86 () ;

	cycles = nITERATIONS * processor_cycles [CPU] ;

	/**********************************************************************

		1 tick is 1193180, the clock frequency for the timer chip
		1.19318 MHz. We executed "cycles" cycles in "ticks" clock
		ticks.  This gives an elapsed time of "cycles"
		cycles * 1193180 ticks per sec / "ticks" ticks = cycles/sec

	 *********************************************************************/

	cps = cycles * 1193180L / ticks ;

	/**********************************************************************

		Note that cycles can't exceed 3599 or the multiply will
		overflow 32 bits. If this would be converted over to use
		32-bit i386 instructions

			mov	eax,1193180
			mul	cycles
			xor	ecx,ecx		; zero out high word of ECX
			mov	cx,ticks
			div	ecx
	
		then this problem could be avoided.  This means that for
		the IDIV CX and a i386/i486 processor, the nITERATIONS can't
		exceed 133 for an 8086, nITERATIONS can't exceed 21, so I
		chose 20.

	 *********************************************************************/

	return (cps / 1000000L) ;
}
